import unittest
import logging

from abduct import captured, out, err

from caravan.tests.util import InTempDir
from caravan.commands.base import BaseCommand


class TestCommand(BaseCommand):

    description = 'DESCRIPTION TEST'

    def setup_arguments(self, parser):
        parser.add_argument('-o', '--opt')

    def run(self):
        arguments = self.args.__dict__.items()
        lines = ['%s=%s' % (k, v) for k, v in arguments]
        lines.append('EndOfOutput')
        return '\n'.join(lines)


class RequiredArgCommand(TestCommand):

    def setup_arguments(self, parser):
        parser.add_argument('-r', '--req', required=True)
        parser.add_argument('-o', '--opt')


class KeyboardInterruptCommand(TestCommand):

    def run(self):
        raise KeyboardInterrupt()


CONFIG_FILE_DATA = """
[caravan]
req = REQFILE
opt = OPTFILE
"""

LOGGING_CONFIG_FILE_DATA = """
[loggers]
keys = root

[logger_root]
level = DEBUG
handlers = console

[handlers]
keys = console

[handler_console]
class = StreamHandler
args = (sys.stderr,)
level = DEBUG
formatter = console

[formatters]
keys = console

[formatter_console]
format = %(message)s
"""


class Test(unittest.TestCase):

    def test_args_optional(self):
        with captured(out(), err()) as (stdout, stderr):
            response = TestCommand.main(args=[])
        self.assertIn('opt=None', response)

        with captured(out(), err()) as (stdout, stderr):
            response = TestCommand.main(args=['--opt', 'OPT'])
        self.assertIn('opt=OPT', response)

    def test_args_missing(self):
        with captured(out(), err()) as (stdout, stderr):
            with self.assertRaises(SystemExit):
                RequiredArgCommand.main(args=[])

        # Py2 and Py3 argparse have different messages
        self.assertIn('error: ', stderr.getvalue())
        self.assertIn('-r/--req', stderr.getvalue())
        self.assertIn('required', stderr.getvalue())

    def test_args_logging_level(self):
        with captured(out(), err()) as (stdout, stderr):
            response = TestCommand.main(args=[])
        self.assertIn('logging_level=%s' % logging.WARNING, response)

        with captured(out(), err()) as (stdout, stderr):
            response = TestCommand.main(args=['--verbose'])
        self.assertIn('logging_level=%s' % logging.INFO, response)

        with captured(out(), err()) as (stdout, stderr):
            response = TestCommand.main(args=['--debug'])
        self.assertIn('logging_level=%s' % logging.DEBUG, response)

    def test_config(self):
        with InTempDir():
            with open('config.conf', 'w') as fd:
                fd.write(CONFIG_FILE_DATA)

            args = ['--config', 'config.conf']
            with captured(out(), err()) as (stdout, stderr):
                TestCommand.main(args=args)

        self.assertIn('EndOfOutput', stdout.getvalue())

    def test_args_logging_config(self):
        with InTempDir():
            with open('logging.conf', 'w') as fd:
                fd.write(LOGGING_CONFIG_FILE_DATA)

            args = ['--logging-config', 'logging.conf']
            with captured(out(), err()) as (stdout, stderr):
                TestCommand.main(args=args)

        self.assertIn('EndOfOutput', stdout.getvalue())

    def test_args_help(self):
        with captured(out(), err()) as (stdout, stderr):
            with self.assertRaises(SystemExit):
                TestCommand.main(args=['--help'])
        self.assertIn(TestCommand.description, stdout.getvalue())

    def test_keyboard_interrupt(self):
        with captured(out(), err()) as (stdout, stderr):
            with self.assertRaises(SystemExit) as exc:
                KeyboardInterruptCommand.main(args=[])
        self.assertEqual(str(exc.exception), '1')



import os
import socket


IDENTITY_SIZE = 256


def get_default_identity():
    identity = "%s-%s" % (socket.getfqdn(), os.getpid())
    return identity[-IDENTITY_SIZE:]  # keep the most important part



from caravan.commands import run_swf_command
from caravan.commands.base import BaseCommand


class Command(BaseCommand):

    description = 'Start a workflow execution'

    def setup_arguments(self, parser):
        parser.add_argument('-d', '--domain', required=True)
        parser.add_argument('-i', '--id', required=True)
        parser.add_argument('-n', '--name', required=True)
        parser.add_argument('-v', '--version', required=True)
        parser.add_argument('-t', '--task-list')

        parser.add_argument(
            '--input',
            help='Raw input data for this execution',
            metavar='"<Some data>"')
        parser.add_argument(
            '--tag-list',
            help='List of tags to associate with the workflow execution',
            metavar='"tag1 tag2 ..."')
        parser.add_argument(
            '--execution-timeout',
            help='The maximum total duration for this workflow execution '
                 '(seconds)')
        parser.add_argument(
            '--task-timeout',
            help='The maximum duration of decision tasks (seconds)')
        parser.add_argument(
            '--child-policy',
            choices=self.CHILD_POLICIES,
            help='Policy to use for the child executions of this execution')
        parser.add_argument(
            '--lambda-role',
            help='ARN of an IAM role that authorizes SWF to invoke Lambda '
                 'functions')

    def run(self):
        workflow_type = {'name': self.args.name, 'version': self.args.version}

        if self.args.task_list is not None:
            task_list = {'name': self.args.task_list}
        else:
            task_list = None

        if self.args.tag_list:
            tag_list = self.args.tag_list.split()
        else:
            tag_list = None

        response = run_swf_command(
            'start_workflow_execution',
            domain=self.args.domain,
            workflowId=self.args.id,
            workflowType=workflow_type,
            input=self.args.input,
            taskList=task_list,
            tagList=tag_list,
            executionStartToCloseTimeout=self.args.execution_timeout,
            taskStartToCloseTimeout=self.args.task_timeout,
            childPolicy=self.args.child_policy,
            lambdaRole=self.args.lambda_role,
            )

        return 'Execution started. RunId: %s' % response['runId']



import json
import unittest

import httpretty
from abduct import captured, out, err

from caravan.commands.signal import Command


class Test(unittest.TestCase):

    @httpretty.activate
    def test_nominal(self):
        args = ['-d', 'DOMAIN', '-i', 'ID', '-s', 'SIG']

        headers = {
            'x-amzn-RequestId': 'd68969c7-3f0d-11e1-9b11-7182192d0b57',
            }
        httpretty.register_uri(httpretty.POST,
                               "https://swf.us-east-1.amazonaws.com/",
                               content_type='application/json',
                               adding_headers=headers,
                               body='')

        with captured(out(), err()) as (stdout, stderr):
            Command.main(args=args)

        request = httpretty.last_request()
        self.assertEqual(request.headers.get('x-amz-target'),
                         'SimpleWorkflowService.SignalWorkflowExecution')

        expected = {
            "domain": "DOMAIN",
            "workflowId": "ID",
            "signalName": "SIG"
            }
        self.assertEqual(json.loads(request.parsed_body), expected)

        self.assertIn('Signal sent.', stdout.getvalue())



import os
import tempfile


class InTempDir(object):

    def __enter__(self):
        self._tmpdir = os.path.realpath(tempfile.mkdtemp())
        self._pwd = os.getcwd()
        os.chdir(self._tmpdir)
        return self

    def __exit__(self, *args):
        os.chdir(self._pwd)



import arrow

from caravan.commands import run_swf_command
from caravan.commands.base import BaseCommand


class Command(BaseCommand):

    description = 'List workflow executions'

    def setup_arguments(self, parser):
        present = arrow.utcnow()
        default_oldest = present.replace(days=-1)
        default_latest = present.clone()

        parser.add_argument('-d', '--domain', required=True)
        parser.add_argument('--oldest', default=default_oldest, type=arrow.get)
        parser.add_argument('--latest', default=default_latest, type=arrow.get)

        filter_group = parser.add_mutually_exclusive_group()
        type_group = filter_group.add_argument_group()
        type_group.add_argument('-n', '--name')
        type_group.add_argument('-v', '--version')
        filter_group.add_argument('--tag')
        filter_group.add_argument('--id')

    def run(self):
        time_filter = {
            'oldestDate': self.args.oldest.naive,
            'latestDate': self.args.latest.naive,
            }

        if self.args.name is not None:
            type_filter = {
                'name': self.args.name,
                }
            if self.args.version is not None:
                type_filter['version'] = self.args.version
        else:
            type_filter = None

        if self.args.tag is not None:
            tag_filter = {
                'tag': self.args.tag,
                }
        else:
            tag_filter = None

        if self.args.id is not None:
            execution_filter = {
                'workflowId': self.args.id,
                }
        else:
            execution_filter = None

        response = run_swf_command(
            'list_open_workflow_executions',
            domain=self.args.domain,
            startTimeFilter=time_filter,
            typeFilter=type_filter,
            tagFilter=tag_filter,
            executionFilter=execution_filter,
            )

        return response['executionInfos']

    def formatter(self, e):
        w = e['workflowType']
        return {
            'Start': arrow.get(e['startTimestamp']).humanize(),
            'Workflow Type': '%s(%s)' % (w['name'], w['version']),
            'Workflow Id': e['execution']['workflowId'],
            'Run Id': e['execution']['runId'],
            'Status': e['executionStatus'],
            }




class Workflow(object):
    """Base implementation of a workflow."""

    name = None
    version = None
    description = None
    defaults = {}

    def __init__(self, task):
        self.task = task

    def __repr__(self):
        return '<Workflow %s(%s)>' % (self.name, self.version)

    def run(self):
        raise NotImplementedError()



from caravan.commands import run_swf_command
from caravan.commands.base import BaseCommand


class Command(BaseCommand):

    description = 'Register a domain'
    default_config_section = 'caravan:domain'

    def setup_arguments(self, parser):
        parser.add_argument('-n', '--name', required=True)
        parser.add_argument(
            '--retention-days',
            dest='retention_days',
            required=True,
            help='Retention period for workflow execution')
        parser.add_argument('--description')

    def run(self):
        run_swf_command(
            'register_domain',
            name=self.args.name,
            workflowExecutionRetentionPeriodInDays=self.args.retention_days,
            description=self.args.description,
            )



from caravan.commands import run_swf_command
from caravan.commands.base import BaseCommand


class Command(BaseCommand):

    description = 'Terminate a workflow execution'

    def setup_arguments(self, parser):
        parser.add_argument('-d', '--domain', required=True)
        parser.add_argument('-i', '--id', required=True)
        parser.add_argument('--run-id')
        parser.add_argument('--reason')
        parser.add_argument('--details')
        parser.add_argument('--child-policy', choices=self.CHILD_POLICIES)

    def run(self):
        run_swf_command(
            'terminate_workflow_execution',
            domain=self.args.domain,
            workflowId=self.args.id,
            runId=self.args.run_id,
            reason=self.args.reason,
            details=self.args.details,
            childPolicy=self.args.child_policy,
            )
        return 'Execution terminated.'



import importlib
import inspect
import sys

import boto3
from botocore.exceptions import ClientError
from botocore.client import Config


def get_swf_connection():
    # Must increase the http timeout since SWF has a timeout of 60 sec
    config = Config(connect_timeout=50, read_timeout=70)
    connection = boto3.client("swf", config=config)
    return connection


def is_response_success(response):
    status_code = response.get('ResponseMetadata', {}).get('HTTPStatusCode')
    return status_code == 200


def run_swf_command(command, **kwargs):
    connection = get_swf_connection()
    command = getattr(connection, command)

    callargs = {k: v for k, v in kwargs.items() if v is not None}

    try:
        response = command(**callargs)
    except ClientError as err:
        sys.exit(err)
    else:
        if is_response_success(response):
            response.pop('ResponseMetadata')
            return response
        else:
            sys.exit('Error: %s' % response)


def find_classes(module, cls):
    def predicate(obj):
        return inspect.isclass(obj) and issubclass(obj, cls) and obj is not cls
    members = inspect.getmembers(module, predicate)
    return [obj for name, obj in members]


class ClassesLoaderFromModule(object):

    """Load and return classes in a module that inherit from a class.

    This module must be in the python sys path.
    """

    def __init__(self, cls):
        self.cls = cls

    def __repr__(self):
        return '<ClassesLoader(%s)>' % self.cls.__name__

    def __call__(self, arg):
        sys.path.append('.')
        try:
            module = importlib.import_module(arg)
        finally:
            sys.path.pop()

        classes = find_classes(module, self.cls)
        if not classes:
            raise ValueError("No workflow in module %s" % arg)

        return classes



from __future__ import print_function
import pprint


def normalize_attribute_key(payload):
    key = [k for k in payload.keys() if k.endswith("Attributes")][0]
    payload['attributes'] = payload.pop(key)
    return payload


class DecisionDone(Exception):
    pass


class WorkflowFailure(Exception):

    def __init__(self, reason, details):
        self.reason = reason
        self.details = details
        super(WorkflowFailure, self).__init__("%s (%s)" % (reason, details))


class DecisionTask(object):

    def __init__(self, data):
        self.events = [normalize_attribute_key(e) for e in data['events']]

        self.token = data['taskToken']
        self.workflow_id = data['workflowExecution']['workflowId']
        self.run_id = data['workflowExecution']['runId']
        self.workflow_type = data['workflowType']['name']
        self.workflow_version = data['workflowType']['version']

        self.decisions = []

    def __repr__(self):
        return '<DecisionTask %s v=%s id=%s runId=%s>' % (
            self.workflow_type,
            self.workflow_version,
            self.workflow_id,
            self.run_id)

    def print_events(self):
        for e in self.events:
            if 'Decision' in e['eventType']:
                continue
            print("==== %3i %s" % (e['eventId'], e['eventType']))
            pprint.pprint(e['attributes'])

    def add_decision(self, decision_type, **attributes):
        attribute_name = '%s%sDecisionAttributes' % (decision_type[0].lower(),
                                                     decision_type[1:])
        decision_payload = {
            'decisionType': decision_type,
            attribute_name: attributes,
        }
        self.decisions.append(decision_payload)

    def fail(self, reason, details='-'):
        raise WorkflowFailure(reason, details)

    def decision_done(self, msg='-'):
        raise DecisionDone('Decision Done: %s' % msg)



import unittest
import json

import httpretty
from abduct import captured, out, err

from caravan.commands.terminate import Command


def httpretty_register():
    headers = {
        'x-amzn-RequestId': 'd68969c7-3f0d-11e1-9b11-7182192d0b57',
        }
    body = '{"runId": "1e536162-f1ea-48b0-85f3-aade88eef2f7"}'
    httpretty.register_uri(httpretty.POST,
                           "https://swf.us-east-1.amazonaws.com/",
                           content_type='application/json',
                           adding_headers=headers,
                           body=body)


def run_command(additional_args):
    httpretty_register()

    args = ['-d', 'DOMAIN', '-i', 'ID']
    args += additional_args
    with captured(out(), err()) as (stdout, stderr):
        Command.main(args=args)

    return stdout, stderr


class Test(unittest.TestCase):

    @httpretty.activate
    def test_nominal(self):
        httpretty_register()
        stdout, stderr = run_command([])
        request = httpretty.last_request()

        self.assertEqual(request.headers.get('x-amz-target'),
                         'SimpleWorkflowService.TerminateWorkflowExecution')

        expected = {
            "domain": "DOMAIN",
            "workflowId": "ID",
            }
        self.assertEqual(json.loads(request.parsed_body), expected)

        self.assertIn('Execution terminated.', stdout.getvalue())

    @httpretty.activate
    def test_parameter_run_id(self):
        httpretty_register()
        run_command(['--run-id', 'RUNID'])
        request = httpretty.last_request()

        expected = {
            "domain": "DOMAIN",
            "runId": 'RUNID',
            "workflowId": "ID",
            }
        self.assertEqual(json.loads(request.parsed_body), expected)

    @httpretty.activate
    def test_parameter_reason(self):
        httpretty_register()
        run_command(['--reason', 'REASON'])
        request = httpretty.last_request()

        expected = {
            "domain": "DOMAIN",
            "reason": 'REASON',
            "workflowId": "ID",
            }
        self.assertEqual(json.loads(request.parsed_body), expected)

    @httpretty.activate
    def test_parameter_details(self):
        httpretty_register()
        run_command(['--details', 'DETAILS'])
        request = httpretty.last_request()

        expected = {
            "domain": "DOMAIN",
            "details": 'DETAILS',
            "workflowId": "ID",
            }
        self.assertEqual(json.loads(request.parsed_body), expected)

    @httpretty.activate
    def test_parameter_child_policy(self):
        test_policy = Command.CHILD_POLICIES[0]

        httpretty_register()
        run_command(['--child-policy', test_policy])
        request = httpretty.last_request()

        expected = {
            "domain": "DOMAIN",
            "childPolicy": test_policy,
            "workflowId": "ID",
            }
        self.assertEqual(json.loads(request.parsed_body), expected)



import logging

from botocore.exceptions import ClientError

from caravan import Workflow
from caravan.commands.base import BaseCommand
from caravan.commands import ClassesLoaderFromModule, get_swf_connection
from caravan.workers.decider import Worker


log = logging.getLogger(__name__)


class Command(BaseCommand):

    description = 'Decider worker'

    def setup_arguments(self, parser):
        parser.add_argument('-m', '--modules',
                            type=ClassesLoaderFromModule(Workflow),
                            nargs='+',
                            required=True)
        parser.add_argument('-d', '--domain',
                            required=True)
        parser.add_argument('-t', '--task-list',
                            required=True)
        parser.add_argument('--register-workflows', action='store_true')

    def run(self):
        connection = get_swf_connection()
        workflows = [w for module in self.args.modules for w in module]

        if self.args.register_workflows:
            log.info("Registering workflow types")
            for workflow in workflows:
                created = register_workflow(connection=connection,
                                            domain=self.args.domain,
                                            workflow=workflow)
                if created:
                    log.info("Workflow type %s: registered.", workflow.name)
                else:
                    log.info("Workflow type %s: already registered.",
                             workflow.name)

        log.info("Start decider worker...")
        worker = Worker(connection=connection,
                        domain=self.args.domain,
                        task_list=self.args.task_list,
                        workflows=workflows)

        while True:
            try:
                worker.run()
            except Exception:  # Doesn't catch KeyboardInterrupt
                log.exception("Decider crashed!")


def register_workflow(connection, domain, workflow):
    args = dict([('default%s' % k, v) for k, v in workflow.defaults.items()])
    if 'defaultTaskList' in args:
        args['defaultTaskList'] = {'name': args['defaultTaskList']}
    description = getattr(workflow, 'description', None)
    if description:
        args['description'] = description

    try:
        connection.register_workflow_type(
            domain=domain,
            name=workflow.name,
            version=workflow.version,
            **args
            )

    except ClientError as err:
        error_code = err.response['Error']['Code']
        if error_code == 'TypeAlreadyExistsFault':
            return False  # Ignore this error
        raise

    return True



import os

import boto3


def setUpModule():
    # Setup Fake AWS Keys (this takes precedence over config file)
    os.environ['AWS_ACCESS_KEY_ID'] = 'AK000000000000000000'
    os.environ['AWS_SECRET_ACCESS_KEY'] = (
        '0000000000000000000000000000000000000000')
    os.environ['AWS_DEFAULT_REGION'] = 'us-east-1'

    # Reset previously created default session (for credentials)
    boto3.setup_default_session()



import unittest
import json

import httpretty
from abduct import captured, out, err
from freezegun import freeze_time

from caravan.commands.list import Command


def httpretty_register():
    headers = {
        'x-amzn-RequestId': 'd68969c7-3f0d-11e1-9b11-7182192d0b57',
        }
    body = """
        { "executionInfos": [
                { "cancelRequested": false,
                  "execution": {
                    "runId": "f5ebbac6-941c-4342-ad69-dfd2f8be6689",
                    "workflowId": "20110927-T-1"
                  },
                  "executionStatus": "OPEN",
                  "startTimestamp": 1326585031.619,
                  "tagList": [
                    "music purchase", "digital", "ricoh-the-dog"
                  ],
                  "workflowType": {
                    "name": "customerOrderWorkflow",
                    "version": "1.0"
                  }
                } ]
            }"""
    httpretty.register_uri(httpretty.POST,
                           "https://swf.us-east-1.amazonaws.com/",
                           content_type='application/json',
                           adding_headers=headers,
                           body=body)


def run_command(additional_args):
    httpretty_register()

    args = ['-d', 'DOMAIN']
    args += additional_args
    with captured(out(), err()) as (stdout, stderr):
        Command.main(args=args)

    return stdout, stderr


class Test(unittest.TestCase):

    @freeze_time("2015-01-01")
    @httpretty.activate
    def test_nominal(self):
        httpretty_register()
        stdout, stderr = run_command([])
        request = httpretty.last_request()

        self.assertEqual(request.headers.get('x-amz-target'),
                         'SimpleWorkflowService.ListOpenWorkflowExecutions')

        expected = {
            "domain": "DOMAIN",
            'startTimeFilter': {
                'latestDate': 1420070400,
                'oldestDate': 1419984000
                }
            }
        self.assertEqual(json.loads(request.parsed_body), expected)

        self.assertIn('.', stdout.getvalue())

    @freeze_time("2015-01-01")
    @httpretty.activate
    def test_parameter_type_filter(self):
        httpretty_register()
        run_command(['-n', 'NAME', '-v', 'VERSION'])
        request = httpretty.last_request()

        expected = {
            "domain": "DOMAIN",
            'startTimeFilter': {
                'latestDate': 1420070400,
                'oldestDate': 1419984000
                },
            'typeFilter': {'name': 'NAME', 'version': 'VERSION'}
            }
        self.assertEqual(json.loads(request.parsed_body), expected)

    @freeze_time("2015-01-01")
    @httpretty.activate
    def test_parameter_tag_filter(self):
        httpretty_register()
        run_command(['--tag', 'TAG'])
        request = httpretty.last_request()

        expected = {
            "domain": "DOMAIN",
            'startTimeFilter': {
                'latestDate': 1420070400,
                'oldestDate': 1419984000
                },
            'tagFilter': {'tag': 'TAG'},
            }
        self.assertEqual(json.loads(request.parsed_body), expected)

    @freeze_time("2015-01-01")
    @httpretty.activate
    def test_parameter_id_filter(self):
        httpretty_register()
        run_command(['--id', 'ID'])
        request = httpretty.last_request()

        expected = {
            "domain": "DOMAIN",
            'startTimeFilter': {
                'latestDate': 1420070400,
                'oldestDate': 1419984000
                },
            'executionFilter': {'workflowId': 'ID'},
            }
        self.assertEqual(json.loads(request.parsed_body), expected)



from caravan.commands import run_swf_command
from caravan.commands.base import BaseCommand


class Command(BaseCommand):

    description = 'Signal a workflow execution'

    def setup_arguments(self, parser):
        parser.add_argument('-d', '--domain', required=True)
        parser.add_argument('-i', '--id', required=True)
        parser.add_argument('--run-id')
        parser.add_argument('-s', '--signal', required=True)
        parser.add_argument(
            '--input',
            help='Raw input data for this signal',
            metavar='"<Some data>"')

    def run(self):
        run_swf_command(
            'signal_workflow_execution',
            domain=self.args.domain,
            workflowId=self.args.id,
            signalName=self.args.signal,
            runId=self.args.run_id,
            input=self.args.input,
            )
        return 'Signal sent.'



from setuptools import setup, find_packages


setup(
    name='caravan',
    version='0.2.0',
    description='Light python framework for AWS SWF',
    long_description=open('README.rst').read(),
    keywords='AWS SWF workflow distributed background task',
    author='Pior Bastida',
    author_email='pior@pbastida.net',
    url='https://github.com/pior/caravan',
    license='MIT',
    packages=find_packages(),
    zip_safe=False,
    install_requires=[
        'boto3',
        'arrow',
        'six',
        'tabulate',
        ],
    extras_require={
        'dev': [
            'ipdb',

            'nose',
            'nosexcover',
            'coverage',
            'yanc == 0.2.4',
            'mock',
            'freezegun',
            'abduct',
            'httpretty',

            'zest.releaser[recommended]',
            'pylama',
            ],
        },
    entry_points={
        'console_scripts': [
            'caravan-decider = caravan.commands.decider:Command.main',
            'caravan-start = caravan.commands.start:Command.main',
            'caravan-signal = caravan.commands.signal:Command.main',
            'caravan-terminate = caravan.commands.terminate:Command.main',
            'caravan-list = caravan.commands.list:Command.main',
            'caravan-domain-register = caravan.commands.domain_register:Command.main',
            ]
        },
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Topic :: Internet',
        'Programming Language :: Python :: 2.7',
        # 'Programming Language :: Python :: 3.4',
        # 'Programming Language :: Python :: 3.5'
        ],
    )



import logging

from botocore.exceptions import ClientError

from caravan.workers import get_default_identity
from caravan.models.decision import DecisionTask, DecisionDone, WorkflowFailure

log = logging.getLogger(__name__)


class Worker(object):

    def __init__(self, connection, domain, task_list, workflows):
        self.conn = connection
        self.domain = domain
        self.task_list = task_list
        self.Workflows = {(w.name, w.version): w for w in workflows}
        self.identity = get_default_identity()

    def run(self):
        log.info('Waiting for a decision task...')
        task = self.poll()
        if task:
            self.run_task(task)

    def poll(self):
        task_list = dict(name=self.task_list)

        resp = self.conn.poll_for_decision_task(domain=self.domain,
                                                taskList=task_list,
                                                identity=self.identity)
        if 'taskToken' not in resp:
            return  # Polling timed out.

        next_page_token = resp.get('nextPageToken')
        while next_page_token:
            page = self.conn.poll_for_decision_task(
                domain=self.domain,
                taskList=task_list,
                identity=self.identity,
                nextPageToken=next_page_token)
            next_page_token = page.get('nextPageToken')
            resp['events'].extend(page['events'])

        return DecisionTask(resp)

    def run_task(self, task):
        log.info('Got a %r', task)
        workflow_key = (task.workflow_type, task.workflow_version)
        workflow_class = self.Workflows.get(workflow_key)

        if not workflow_class:
            log.warning('Unknown workflow %s', task)
            return

        log.info('Running %r...', workflow_class)
        try:
            workflow_class(task).run()
        except DecisionDone as done:
            log.info(str(done))
        except WorkflowFailure as failure:
            task.add_decision('FailWorkflowExecution',
                              reason=failure.reason,
                              details=failure.details)

        log.info('Respond with decisions: %s', task.decisions)
        self.respond_decision(task)

    def respond_decision(self, task):
        try:
            self.conn.respond_decision_task_completed(
                taskToken=task.token,
                decisions=task.decisions)
        except ClientError as exc:
            log.error('Decision response failed for %s: %s\n%s',
                      task, exc.message, exc.response)
        except Exception as exc:
            log.error('Decision response failed for %s: %s', task, exc)



from __future__ import print_function
import argparse
from six.moves import configparser
import logging
import logging.config
import sys

import six
from tabulate import tabulate


class BaseCommand(object):

    """Base class for command-line tools."""

    default_config_section = 'caravan'

    CHILD_POLICIES = [
        'TERMINATE',
        'REQUEST_CANCEL',
        'ABANDON',
        ]

    @classmethod
    def main(cls, args=None):
        """Setuptools console-script entrypoint"""
        cmd = cls()
        cmd._parse_args(args=args)
        cmd._setup_logging()
        response = cmd._run()
        output = cmd._handle_response(response)
        if output is not None:
            print(output)
        return response

    def _parse_args(self, args=None):
        # Config only parser
        config_parser = argparse.ArgumentParser(description=self.description,
                                                add_help=False)
        config_parser.add_argument('-c', '--config',
                                   help='config file for setup.')
        config_parser.add_argument('--config-section',
                                   default=self.default_config_section,
                                   help='section of the config file for '
                                        'setup.')
        config_args, remaining_args = config_parser.parse_known_args(args=args)

        # Full parser
        parser = argparse.ArgumentParser(description=self.description,
                                         parents=[config_parser])
        self._setup_base_arguments(parser)
        self.setup_arguments(parser)

        # Read defaults from config file
        if config_args.config:
            cp = configparser.RawConfigParser()
            with open(config_args.config) as fp:
                cp.readfp(fp)
            config_items = cp.items(config_args.config_section)

            valid_options = [option_string
                             for action in parser._actions
                             for option_string in action.option_strings]
            nargs_options = {option_string: action.nargs
                             for action in parser._actions
                             for option_string in action.option_strings}

            for option, value in config_items:
                option_string = '--%s' % option

                if option_string in valid_options:
                    if nargs_options.get(option_string) == '+':
                        value = value.split()
                        option_args = [option_string] + value
                    else:
                        option_args = [option_string, value]
                    args.extend(option_args)

        self.args = parser.parse_args(args)

    def _setup_base_arguments(self, parser):
        parser.add_argument('--logging-config',
                            dest='logging_config',
                            help='Optional config file for logging.'
                                 ' Default to config_uri')
        parser.add_argument('--verbose',
                            dest='logging_level',
                            default=logging.WARNING,
                            action='store_const',
                            const=logging.INFO)
        parser.add_argument('--debug',
                            dest='logging_level',
                            default=logging.WARNING,
                            action='store_const',
                            const=logging.DEBUG)

    def _setup_logging(self):
        if self.args.logging_config:
            logging.config.fileConfig(self.args.logging_config)
        elif self.args.config:
            try:
                logging.config.fileConfig(self.args.config)
            except (configparser.NoSectionError, KeyError):
                pass
        else:
            logging.basicConfig(level=self.args.logging_level)

    def _run(self):
        logging.debug('Run command with args: %s', self.args)
        try:
            return self.run()
        except KeyboardInterrupt:
            sys.exit(1)

    def _handle_response(self, response):
        if response is None:
            return "Success."
        elif isinstance(response, six.string_types):
            return response
        elif isinstance(response, list):
            if len(response) == 0:
                return 'No results.'
            if hasattr(self, 'formatter'):
                response = map(self.formatter, response)
            return tabulate(response, headers='keys', tablefmt="plain")



import unittest
import json

import httpretty
from abduct import captured, out, err

from caravan.commands.start import Command


def httpretty_register():
    headers = {
        'x-amzn-RequestId': 'd68969c7-3f0d-11e1-9b11-7182192d0b57',
        }
    body = '{"runId": "1e536162-f1ea-48b0-85f3-aade88eef2f7"}'
    httpretty.register_uri(httpretty.POST,
                           "https://swf.us-east-1.amazonaws.com/",
                           content_type='application/json',
                           adding_headers=headers,
                           body=body)


def run_command(additional_args):
    httpretty_register()

    args = ['-d', 'DOMAIN', '-n', 'NAME', '-v', 'VERSION', '-i', 'ID']
    args += additional_args
    with captured(out(), err()) as (stdout, stderr):
        Command.main(args=args)

    return stdout, stderr


class Test(unittest.TestCase):

    @httpretty.activate
    def test_nominal(self):
        httpretty_register()
        stdout, stderr = run_command([])
        request = httpretty.last_request()

        self.assertEqual(request.headers.get('x-amz-target'),
                         'SimpleWorkflowService.StartWorkflowExecution')

        expected = {
            "domain": "DOMAIN",
            "workflowType": {"version": "VERSION", "name": "NAME"},
            "workflowId": "ID",
            }
        self.assertEqual(json.loads(request.parsed_body), expected)

        self.assertIn('Execution started.', stdout.getvalue())
        self.assertIn('1e536162-f1ea-48b0-85f3-aade88eef2f7',
                      stdout.getvalue())

    @httpretty.activate
    def test_parameter_task_list(self):
        httpretty_register()
        run_command(['-t', 'TASKLISTNAME'])
        request = httpretty.last_request()

        expected = {
            "domain": "DOMAIN",
            "taskList": {"name": "TASKLISTNAME"},
            "workflowId": "ID",
            "workflowType": {"version": "VERSION", "name": "NAME"},
            }
        self.assertEqual(json.loads(request.parsed_body), expected)

    @httpretty.activate
    def test_parameter_tag(self):
        httpretty_register()
        run_command(['--tag', 'TAG'])
        request = httpretty.last_request()

        expected = {
            "domain": "DOMAIN",
            "workflowType": {"version": "VERSION", "name": "NAME"},
            "workflowId": "ID",
            "tagList": ["TAG"],
            }
        self.assertEqual(json.loads(request.parsed_body), expected)

    @httpretty.activate
    def test_parameter_tag_two(self):
        httpretty_register()
        run_command(['--tag', 'TAG1 TAG2'])
        request = httpretty.last_request()

        expected = {
            "domain": "DOMAIN",
            "workflowType": {"version": "VERSION", "name": "NAME"},
            "workflowId": "ID",
            "tagList": ["TAG1", "TAG2"]
            }
        self.assertEqual(json.loads(request.parsed_body), expected)

    @httpretty.activate
    def test_parameter_input(self):
        httpretty_register()
        run_command(['--input', 'INPUT'])
        request = httpretty.last_request()

        expected = {
            "input": "INPUT",
            "domain": "DOMAIN",
            "workflowType": {"version": "VERSION", "name": "NAME"},
            "workflowId": "ID"
            }
        self.assertEqual(json.loads(request.parsed_body), expected)

    @httpretty.activate
    def test_parameter_execution_timeout(self):
        httpretty_register()
        run_command(['--execution-timeout', '60'])
        request = httpretty.last_request()

        expected = {
            "domain": "DOMAIN",
            "workflowType": {"version": "VERSION", "name": "NAME"},
            "workflowId": "ID",
            "executionStartToCloseTimeout": "60",
            }
        self.assertEqual(json.loads(request.parsed_body), expected)

    @httpretty.activate
    def test_parameter_task_timeout(self):
        httpretty_register()
        run_command(['--task-timeout', '60'])
        request = httpretty.last_request()

        expected = {
            "domain": "DOMAIN",
            "workflowType": {"version": "VERSION", "name": "NAME"},
            "workflowId": "ID",
            "taskStartToCloseTimeout": "60",
            }
        self.assertEqual(json.loads(request.parsed_body), expected)

    @httpretty.activate
    def test_parameter_child_policy(self):
        test_policy = Command.CHILD_POLICIES[0]
        httpretty_register()
        run_command(['--child-policy', test_policy])
        request = httpretty.last_request()

        Command.CHILD_POLICIES[0]
        expected = {
            "domain": "DOMAIN",
            "workflowType": {"version": "VERSION", "name": "NAME"},
            "workflowId": "ID",
            "childPolicy": test_policy,
            }
        self.assertEqual(json.loads(request.parsed_body), expected)

    @httpretty.activate
    def test_parameter_lambda_role(self):
        httpretty_register()
        run_command(['--lambda-role', 'ROLE'])
        request = httpretty.last_request()

        Command.CHILD_POLICIES[0]
        expected = {
            "domain": "DOMAIN",
            "workflowType": {"version": "VERSION", "name": "NAME"},
            "workflowId": "ID",
            "lambdaRole": 'ROLE',
            }
        self.assertEqual(json.loads(request.parsed_body), expected)
